## How to use `metadocs`


This Documentation has 2 layers:

* Each Python Project listed in your Home's Projects section has its own documentation, independantly generated _via_ [**Sphinx**](http://www.sphinx-doc.org/en/master/), especially thanks to `autodoc` which generates documentation from docstrings in the source code
</br></br>

* On top of these Project Documentations is the present Home Documentation which you are reading. It is generated by [**Mkdocs**](http://www.mkdocs.org) and is used to explore projects and store general, shared Documentation accross projects


## How to Use

### mkdocs

    mkdocs.yml    # The configuration file for mkdocs.
    docs/
        index.md  # The documentation homepage (this one).
        ...       # Other markdown pages, images and other files. Standard mkdocs.

!!! important
    `index.md` **has** to have a section "# Projects"


### Adding a new project to the list

#### Manually

To list a new project in the *Projects* section:

1. Add it at the root of this Home Documentation (where `docs/` and `mkdocs.yml` are)
2. Mention it in this file's (`index.md`) [Projects](/#projects) section as follows:
```
    * [My Project](/my_awesome_project) - My Project's short description`
```

!!! important
    `metadocs` expects to find a folder named `build/` in your project, and will serve `build/index.html`.

!!! note 
    `my_awesome_project` is the name of the folder your project is in, and the `/` is mandatory

#### Automatically add a python project

`metadocs` comes with a handy `autodoc` feature to:

* create a new `sphinx` documentation
* reference your docstrings with `sphinx apidoc`
* add the project to your Home Documentation

You may now edit the `.rst` files. See [Write Sphinx Documentation](Writing_Sphinx_Documentation)

###### Example

Say you have this file structure:

    mkdocs.yml    # The configuration file for mkdocs.
    docs/
        index.md  # The documentation homepage (this one).
        ...       # Other markdown pages, images and other files. Standard mkdocs.
    project1/
        ... # an existing project's documentation

and say you want to add `project2` to your list. You just have to create a new directory, put your code in there and run `metadocs autodoc`:

```bash
# from your documentation's root
$ mkdir project2
$ cd project 2
$ cp -r path/to/project2 ./
$ pwd
path/to/your/documentation/project2
$ ls
project2
$ metadocs autodoc
... some prints
$ ls
Makefile    source    build    project2
```
now your file structure looks like:

    mkdocs.yml    # The configuration file for mkdocs.
    docs/
        index.md  # The documentation homepage (this one).
        ...       # Other markdown pages, images and other files. Standard mkdocs.
    project1/
        ... # an existing project's documentation
    project2/
        project2/ # your documented code here
            ...
        source/ # the source documents for sphinx to build your doc
            index.rst # rst files hold the code documentation's skeletton
            conf.py # sphinx configuration file
            ...
        build/ # the built docs, in html
            index.html
            ...
        Makefile # sphinx's make instructions; metadocs runs make html


Now you can check your Home Documentation's `Projects` section, it will be there!

### Built-in Server

Running `metadocs serve` will start a local http server for you to browse your documentation. If you don't want to use the default port (`8443`), use the `serve_port` flag: `metadocs serve -s 7777`

!!! note "Changes"
    `metadocs` tracks changes and modifications to `.md`, `.rst` and `.py` files: it will automatically trigger a build to update the html files. Sweet!

### Building the Documentations

If you want to trigger a build manually, run `metadocs build` with some flags:

```
usage: metadocs [-h] [--version] [-v] [-A] [-F] [-o]
             [-p [PROJECTS [PROJECTS ...]]]
             [-m [MOCK_IMPORTS [MOCK_IMPORTS ...]]] [-s [SERVE_PORT]]
             [--offline]
             [{init,build,serve,version,clean,autodoc}] [project_name]

Building Doc

positional arguments:
  {init,build,serve,version,clean,autodoc}
                        Available commands for metadocs
  project_name          [init] Your project's name

optional arguments:
  -h, --help            show this help message and exit
  --version             Print metadocs's version
  -v, --verbose         [build] verbose flag (Sphinx will stay verbose)
  -A, --all             [build] Build doc for all projects
  -F, --force           [build] force the build, no verification asked
  -o, --only_index      [build] only build projects listed in the
                        Documentation's Home
  -p [PROJECTS [PROJECTS ...]], --projects [PROJECTS [PROJECTS ...]]
                        [build] list of projects to build
  -m [MOCK_IMPORTS [MOCK_IMPORTS ...]], --mock_imports [MOCK_IMPORTS [MOCK_IMPORTS ...]]
                        [autodoc] list of imports to mock
  -s [SERVE_PORT], --serve_port [SERVE_PORT]
                        [serve] the server's port, defaults to 8443
  --offline             [build, serve] Whether references to external APIs
                        should be deleted from html files + load material
                        icons locally
```

`metadocs build` does 3 things:

1. Build your projects' documentations by running `make html` in the relevant projects (according to the listed projects if you use `-p` or `-A` flags)
2. Update your projects' documentations' `html` files to add a link back here to the Home Documentation
3. Build `mkdoc`'s home documentation, _i.e._ this very document and others as parametrized in `mkdocs.yml`

A couple examples:

* To build a new project's documentation but leave others untouched:
    * `metadocs build -p my_awesome_project`
* To build documentations for all projects which are listed in [Projects](/#projects):
    * `metadocs build -A -o`
    * There may be projects not listed above for any reason and you may not want to build their documentation, hence the `-o` flag

### Configuration

#### Home Documentation: `mkdocs`

Explore the `mkdocs.yml` file to configure your Home Documentation. Here are the keys:

* `site_name:The name you want to give to this Home Documentation`.
* By default `pages` will contain an alphanumerically sorted, nested list of all the Markdown files found within the `docs` directory and its sub-directories.
    * If you want to specify them yourself, add a key `pages:` (list of the pages available, listed on the sidebar)
    * items in this list are `- page's title: file.md`
* `theme: material` you can also use `readthedocs` or install any other
* `markdown_extensions:` is a list of extensions to use. they are listed [here](https://python-markdown.github.io/extensions/#officially-supported-extensions)
    * `- admonition` allows for reStructuredText-style admonition, that is the "important", "note" and "danger" colored boxes

##### Useful Resources

* [Mkdocs's Getting Started](https://www.mkdocs.org/user-guide/writing-your-docs/)
* [Material for Mkdocs's customization instructions](https://squidfunk.github.io/mkdocs-material/customization/)
* [Material for Mkdocs's supported extensions list](https://squidfunk.github.io/mkdocs-material/extensions/admonition/)


#### Projects' Documentations: `sphinx`

In all python projects' `source/` directory are the source files for `sphinx` to generate your documentation. As for the Home Documentation you should explore the `conf.py` file in this `source` directory to set this particular project's configuration.
